@@ -2,6 +2,7 @@
  * preferences.c
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2019 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -58,6 +59,7 @@
 #define PREF_GROUP_ALIAS "alias"
 #define PREF_GROUP_OTR "otr"
 #define PREF_GROUP_PGP "pgp"
+#define PREF_GROUP_OMEMO "omemo"
 #define PREF_GROUP_MUC "muc"
 #define PREF_GROUP_PLUGINS "plugins"
 
@@ -749,6 +751,96 @@ prefs_get_occupants_size(void)
     }
 }
 
+char
+prefs_get_occupants_char(void)
+{
+    char result = 0;
+
+    char *resultstr = g_key_file_get_string(prefs, PREF_GROUP_UI, "occupants.char", NULL);
+    if (!resultstr) {
+        result =  0;
+    } else {
+        result = resultstr[0];
+    }
+    free(resultstr);
+
+    return result;
+}
+
+void
+prefs_set_occupants_char(char ch)
+{
+    char str[2];
+    str[0] = ch;
+    str[1] = '\0';
+
+    g_key_file_set_string(prefs, PREF_GROUP_UI, "occupants.char", str);
+    _save_prefs();
+}
+
+void
+prefs_clear_occupants_char(void)
+{
+    g_key_file_remove_key(prefs, PREF_GROUP_UI, "occupants.char", NULL);
+    _save_prefs();
+}
+
+gint
+prefs_get_occupants_indent(void)
+{
+    if (!g_key_file_has_key(prefs, PREF_GROUP_UI, "occupants.indent", NULL)) {
+        return 2;
+    }
+
+    gint result = g_key_file_get_integer(prefs, PREF_GROUP_UI, "occupants.indent", NULL);
+    if (result < 0) {
+        result = 0;
+    }
+
+    return result;
+}
+
+void
+prefs_set_occupants_indent(gint value)
+{
+    g_key_file_set_integer(prefs, PREF_GROUP_UI, "occupants.indent", value);
+    _save_prefs();
+}
+
+char
+prefs_get_occupants_header_char(void)
+{
+    char result = 0;
+
+    char *resultstr = g_key_file_get_string(prefs, PREF_GROUP_UI, "occupants.header.char", NULL);
+    if (!resultstr) {
+        result =  0;
+    } else {
+        result = resultstr[0];
+    }
+    free(resultstr);
+
+    return result;
+}
+
+void
+prefs_set_occupants_header_char(char ch)
+{
+    char str[2];
+    str[0] = ch;
+    str[1] = '\0';
+
+    g_key_file_set_string(prefs, PREF_GROUP_UI, "occupants.header.char", str);
+    _save_prefs();
+}
+
+void
+prefs_clear_occupants_header_char(void)
+{
+    g_key_file_remove_key(prefs, PREF_GROUP_UI, "occupants.header.char", NULL);
+    _save_prefs();
+}
+
 void
 prefs_set_roster_size(gint value)
 {
@@ -822,6 +914,33 @@ prefs_set_pgp_char(char ch)
     _save_prefs();
 }
 
+char
+prefs_get_omemo_char(void)
+{
+    char result = '~';
+
+    char *resultstr = g_key_file_get_string(prefs, PREF_GROUP_OMEMO, "omemo.char", NULL);
+    if (!resultstr) {
+        result =  '~';
+    } else {
+        result = resultstr[0];
+    }
+    free(resultstr);
+
+    return result;
+}
+
+void
+prefs_set_omemo_char(char ch)
+{
+    char str[2];
+    str[0] = ch;
+    str[1] = '\0';
+
+    g_key_file_set_string(prefs, PREF_GROUP_OMEMO, "omemo.char", str);
+    _save_prefs();
+}
+
 char
 prefs_get_roster_header_char(void)
 {
@@ -1557,6 +1676,7 @@ _get_group(preference_t pref)
         case PREF_HISTORY:
         case PREF_OCCUPANTS:
         case PREF_OCCUPANTS_JID:
+        case PREF_OCCUPANTS_WRAP:
         case PREF_STATUSES:
         case PREF_STATUSES_CONSOLE:
         case PREF_STATUSES_CHAT:
@@ -1657,6 +1777,9 @@ _get_group(preference_t pref)
             return PREF_GROUP_MUC;
         case PREF_PLUGINS_SOURCEPATH:
             return PREF_GROUP_PLUGINS;
+        case PREF_OMEMO_LOG:
+        case PREF_OMEMO_POLICY:
+            return PREF_GROUP_OMEMO;
         default:
             return NULL;
     }
@@ -1701,6 +1824,8 @@ _get_key(preference_t pref)
             return "occupants";
         case PREF_OCCUPANTS_JID:
             return "occupants.jid";
+        case PREF_OCCUPANTS_WRAP:
+            return "occupants.wrap";
         case PREF_MUC_PRIVILEGES:
             return "privileges";
         case PREF_STATUSES:
@@ -1871,6 +1996,10 @@ _get_key(preference_t pref)
             return "statusbar.chat";
         case PREF_STATUSBAR_ROOM:
             return "statusbar.room";
+        case PREF_OMEMO_LOG:
+            return "log";
+        case PREF_OMEMO_POLICY:
+            return "policy";
         default:
             return NULL;
     }
@@ -1989,6 +2118,10 @@ _get_default_string(preference_t pref)
             return "user";
         case PREF_STATUSBAR_ROOM:
             return "room";
+        case PREF_OMEMO_LOG:
+            return "redact";
+        case PREF_OMEMO_POLICY:
+            return "automatic";
         default:
             return NULL;
     }
