@@ -148,6 +148,9 @@ typedef enum {
     PREF_STATUSBAR_SELF,
     PREF_STATUSBAR_CHAT,
     PREF_STATUSBAR_ROOM,
+    PREF_OMEMO_LOG,
+    PREF_OMEMO_POLICY,
+    PREF_OCCUPANTS_WRAP,
 } preference_t;
 
 typedef struct prof_alias_t {
@@ -216,6 +219,8 @@ char prefs_get_otr_char(void);
 void prefs_set_otr_char(char ch);
 char prefs_get_pgp_char(void);
 void prefs_set_pgp_char(char ch);
+char prefs_get_omemo_char(void);
+void prefs_set_omemo_char(char ch);
 
 char prefs_get_roster_header_char(void);
 void prefs_set_roster_header_char(char ch);
@@ -235,6 +240,12 @@ void prefs_clear_roster_room_char(void);
 char prefs_get_roster_room_private_char(void);
 void prefs_set_roster_room_private_char(char ch);
 void prefs_clear_roster_room_private_char(void);
+char prefs_get_occupants_char(void);
+void prefs_set_occupants_char(char ch);
+void prefs_clear_occupants_char(void);
+char prefs_get_occupants_header_char(void);
+void prefs_set_occupants_header_char(char ch);
+void prefs_clear_occupants_header_char(void);
 
 gint prefs_get_roster_contact_indent(void);
 void prefs_set_roster_contact_indent(gint value);
@@ -242,6 +253,8 @@ gint prefs_get_roster_resource_indent(void);
 void prefs_set_roster_resource_indent(gint value);
 gint prefs_get_roster_presence_indent(void);
 void prefs_set_roster_presence_indent(gint value);
+gint prefs_get_occupants_indent(void);
+void prefs_set_occupants_indent(gint value);
 
 void prefs_add_login(const char *jid);
 
