@@ -2,6 +2,7 @@
  * theme.c
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2019 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -152,6 +153,7 @@ theme_init(const char *const theme_name)
     g_hash_table_insert(defaults, strdup("roster.room.trigger"),     strdup("green"));
     g_hash_table_insert(defaults, strdup("roster.room.mention"),     strdup("green"));
     g_hash_table_insert(defaults, strdup("occupants.header"),        strdup("yellow"));
+    g_hash_table_insert(defaults, strdup("untrusted"),               strdup("red"));
 
     _load_preferences();
 }
@@ -388,6 +390,7 @@ _load_preferences(void)
     _set_boolean_preference("resource.message", PREF_RESOURCE_MESSAGE);
     _set_boolean_preference("occupants", PREF_OCCUPANTS);
     _set_boolean_preference("occupants.jid", PREF_OCCUPANTS_JID);
+    _set_boolean_preference("occupants.wrap", PREF_OCCUPANTS_WRAP);
     _set_boolean_preference("roster", PREF_ROSTER);
     _set_boolean_preference("roster.offline", PREF_ROSTER_OFFLINE);
     _set_boolean_preference("roster.resource", PREF_ROSTER_RESOURCE);
@@ -408,7 +411,7 @@ _load_preferences(void)
     _set_boolean_preference("enc.warn", PREF_ENC_WARN);
     _set_boolean_preference("tls.show", PREF_TLS_SHOW);
     _set_boolean_preference("statusbar.show.name", PREF_STATUSBAR_SHOW_NAME);
-    _set_boolean_preference("statusbar.show.nuumber", PREF_STATUSBAR_SHOW_NUMBER);
+    _set_boolean_preference("statusbar.show.number", PREF_STATUSBAR_SHOW_NUMBER);
 
     _set_string_preference("time.console", PREF_TIME_CONSOLE);
     _set_string_preference("time.chat", PREF_TIME_CHAT);
@@ -452,6 +455,31 @@ _load_preferences(void)
         prefs_set_occupants_size(occupants_size);
     }
 
+    if (g_key_file_has_key(theme, "ui", "occupants.char", NULL)) {
+        gchar *ch = g_key_file_get_string(theme, "ui", "occupants.char", NULL);
+        if (ch && strlen(ch) > 0) {
+            prefs_set_occupants_char(ch[0]);
+            g_free(ch);
+        }
+    } else {
+        prefs_clear_occupants_char();
+    }
+
+    if (g_key_file_has_key(theme, "ui", "occupants.indent", NULL)) {
+        gint occupants_indent = g_key_file_get_integer(theme, "ui", "occupants.indent", NULL);
+        prefs_set_occupants_indent(occupants_indent);
+    }
+
+    if (g_key_file_has_key(theme, "ui", "occupants.header.char", NULL)) {
+        gchar *ch = g_key_file_get_string(theme, "ui", "occupants.header.char", NULL);
+        if (ch && strlen(ch) > 0) {
+            prefs_set_occupants_header_char(ch[0]);
+            g_free(ch);
+        }
+    } else {
+        prefs_clear_occupants_header_char();
+    }
+
     if (g_key_file_has_key(theme, "ui", "roster.size", NULL)) {
         gint roster_size = g_key_file_get_integer(theme, "ui", "roster.size", NULL);
         prefs_set_roster_size(roster_size);
@@ -548,6 +576,14 @@ _load_preferences(void)
         }
     }
 
+    if (g_key_file_has_key(theme, "ui", "omemo.char", NULL)) {
+        gchar *ch = g_key_file_get_string(theme, "ui", "omemo.char", NULL);
+        if (ch && strlen(ch) > 0) {
+            prefs_set_omemo_char(ch[0]);
+            g_free(ch);
+        }
+    }
+
     if (g_key_file_has_key(theme, "ui", "titlebar.position", NULL) &&
             g_key_file_has_key(theme, "ui", "mainwin.position", NULL) &&
             g_key_file_has_key(theme, "ui", "statusbar.position", NULL) &&
@@ -819,6 +855,7 @@ theme_attrs(theme_item_t attrs)
     case THEME_ROSTER_ROOM_TRIGGER:     _theme_prep_fgnd("roster.room.trigger",     lookup_str, &bold); break;
     case THEME_ROSTER_ROOM_MENTION:     _theme_prep_fgnd("roster.room.mention",     lookup_str, &bold); break;
     case THEME_OCCUPANTS_HEADER:        _theme_prep_fgnd("occupants.header",        lookup_str, &bold); break;
+    case THEME_UNTRUSTED:               _theme_prep_fgnd("untrusted",               lookup_str, &bold); break;
     case THEME_WHITE:                   g_string_append(lookup_str, "white");   bold = FALSE;   break;
     case THEME_WHITE_BOLD:              g_string_append(lookup_str, "white");   bold = TRUE;    break;
     case THEME_GREEN:                   g_string_append(lookup_str, "green");   bold = FALSE;   break;
