@@ -37,6 +37,8 @@
 
 #include <glib.h>
 
+#include "xmpp/message.h"
+
 // log levels
 typedef enum {
     PROF_LEVEL_DEBUG,
@@ -71,16 +73,21 @@ void chat_log_init(void);
 void chat_log_msg_out(const char *const barejid, const char *const msg);
 void chat_log_otr_msg_out(const char *const barejid, const char *const msg);
 void chat_log_pgp_msg_out(const char *const barejid, const char *const msg);
+void chat_log_omemo_msg_out(const char *const barejid, const char *const msg);
 
-void chat_log_msg_in(const char *const barejid, const char *const msg, GDateTime *timestamp);
-void chat_log_otr_msg_in(const char *const barejid, const char *const msg, gboolean was_decrypted, GDateTime *timestamp);
-void chat_log_pgp_msg_in(const char *const barejid, const char *const msg, GDateTime *timestamp);
+void chat_log_msg_in(ProfMessage *message);
+void chat_log_otr_msg_in(ProfMessage *message);
+void chat_log_pgp_msg_in(ProfMessage *message);
+void chat_log_omemo_msg_in(ProfMessage *message);
 
 void chat_log_close(void);
 GSList* chat_log_get_previous(const gchar *const login, const gchar *const recipient);
 
 void groupchat_log_init(void);
-void groupchat_log_chat(const gchar *const login, const gchar *const room, const gchar *const nick,
-    const gchar *const msg);
+
+void groupchat_log_msg_out(const gchar *const room, const gchar *const msg);
+void groupchat_log_msg_in(const gchar *const room, const gchar *const nick, const gchar *const msg);
+void groupchat_log_omemo_msg_out(const gchar *const room, const gchar *const msg);
+void groupchat_log_omemo_msg_in(const gchar *const room, const gchar *const nick, const gchar *const msg);
 
 #endif
