@@ -1,7 +1,7 @@
 /*
  * autocompleters.h
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -42,7 +42,7 @@ void autocompleters_add(const char *const plugin_name, const char *key, char **i
 void autocompleters_remove(const char *const plugin_name, const char *key, char **items);
 void autocompleters_clear(const char *const plugin_name, const char *key);
 void autocompleters_filepath_add(const char *const plugin_name, const char *prefix);
-char* autocompleters_complete(const char * const input);
+char* autocompleters_complete(const char * const input, gboolean previous);
 void autocompleters_reset(void);
 void autocompleters_destroy(void);
 
