@@ -2,6 +2,7 @@
  * console.c
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2019 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -412,7 +413,8 @@ cons_about(void)
         }
     }
 
-    win_println(console, THEME_DEFAULT, '-', "Copyright (C) 2012 - 2018 James Booth <%s>.", PACKAGE_BUGREPORT);
+    win_println(console, THEME_DEFAULT, '-', "Copyright (C) 2012 - 2019 James Booth <%s>.", PACKAGE_BUGREPORT);
+    win_println(console, THEME_DEFAULT, '-', "Copyright (C) 2019 Michael Vetter <jubalh@iodoru.org>.");
     win_println(console, THEME_DEFAULT, '-', "License GPLv3+: GNU GPL version 3 or later <https://www.gnu.org/licenses/gpl.html>");
     win_println(console, THEME_DEFAULT, '-', "");
     win_println(console, THEME_DEFAULT, '-', "This is free software; you are free to change and redistribute it.");
@@ -442,7 +444,7 @@ cons_check_version(gboolean not_available_msg)
         if (relase_valid) {
             if (release_is_new(latest_release)) {
                 win_println(console, THEME_DEFAULT, '-', "A new version of Profanity is available: %s", latest_release);
-                win_println(console, THEME_DEFAULT, '-', "Check <http://www.profanity.im> for details.");
+                win_println(console, THEME_DEFAULT, '-', "Check <https://profanity-im.github.io> for details.");
                 win_println(console, THEME_DEFAULT, '-', "");
             } else {
                 if (not_available_msg) {
@@ -1232,8 +1234,28 @@ cons_occupants_setting(void)
     else
         cons_show("Occupant jids (/occupants)          : hide");
 
+    if (prefs_get_boolean(PREF_OCCUPANTS_WRAP))
+        cons_show("Occupants wrap (/occupants)         : ON");
+    else
+        cons_show("Occupants wrap (/occupants)         : OFF");
+
+    char occupants_ch = prefs_get_occupants_char();
+    if (occupants_ch)
+        cons_show("Occupants char (/occupants)         : %c", occupants_ch);
+    else
+        cons_show("Occupants char (/occupants)         : none");
+
+    gint occupant_indent = prefs_get_occupants_indent();
+    cons_show("Occupant indent (/occupants)        : %d", occupant_indent);
+
     int size = prefs_get_occupants_size();
     cons_show("Occupants size (/occupants)         : %d", size);
+
+    char header_ch = prefs_get_occupants_header_char();
+    if (header_ch)
+        cons_show("Occupants header char (/occupants)  : %c", header_ch);
+    else
+        cons_show("Occupants header char (/occupants)  : none");
 }
 
 void
@@ -1284,9 +1306,9 @@ cons_time_setting(void)
 
     char *pref_time_conf = prefs_get_string(PREF_TIME_CONFIG);
     if (g_strcmp0(pref_time_conf, "off") == 0)
-        cons_show("Time config (/time)             : OFF");
+        cons_show("Time config (/time)                 : OFF");
     else
-        cons_show("Time config (/time)             : %s", pref_time_conf);
+        cons_show("Time config (/time)                 : %s", pref_time_conf);
     prefs_free_string(pref_time_conf);
 
     char *pref_time_private = prefs_get_string(PREF_TIME_PRIVATE);
@@ -1998,6 +2020,32 @@ cons_show_pgp_prefs(void)
     cons_alert();
 }
 
+void
+cons_show_omemo_prefs(void)
+{
+    cons_show("OMEMO preferences:");
+    cons_show("");
+
+    char *policy_value = prefs_get_string(PREF_OMEMO_POLICY);
+    cons_show("OMEMO policy (/omemo policy) : %s", policy_value);
+    prefs_free_string(policy_value);
+
+    char *log_value = prefs_get_string(PREF_OMEMO_LOG);
+    if (strcmp(log_value, "on") == 0) {
+        cons_show("OMEMO logging (/omemo log)   : ON");
+    } else if (strcmp(log_value, "off") == 0) {
+        cons_show("OMEMO logging (/omemo log)   : OFF");
+    } else {
+        cons_show("OMEMO logging (/omemo log)   : Redacted");
+    }
+    prefs_free_string(log_value);
+
+    char ch = prefs_get_omemo_char();
+    cons_show("OMEMO char (/omemo char)     : %c", ch);
+
+    cons_alert();
+}
+
 void
 cons_show_themes(GSList *themes)
 {
@@ -2072,6 +2120,8 @@ cons_prefs(void)
     cons_show("");
     cons_show_pgp_prefs();
     cons_show("");
+    cons_show_omemo_prefs();
+    cons_show("");
 
     cons_alert();
 }
