@@ -742,14 +742,12 @@ ui_print_system_msg_from_recipient(const char *const barejid, const char *messag
     ProfChatWin *chatwin = wins_get_chat(barejid);
     ProfWin *window = (ProfWin*)chatwin;
     if (window == NULL) {
-        int num = 0;
         window = wins_new_chat(barejid);
         if (window) {
             chatwin = (ProfChatWin*)window;
-            num = wins_get_num(window);
+            int num = wins_get_num(window);
             status_bar_active(num, WIN_CHAT, chatwin->barejid);
         } else {
-            num = 0;
             window = wins_get_console();
             status_bar_active(1, WIN_CONSOLE, "console");
         }
@@ -763,7 +761,7 @@ ui_room_join(const char *const roomjid, gboolean focus)
 {
     ProfMucWin *mucwin = wins_get_muc(roomjid);
     if (mucwin == NULL) {
-        mucwin = (ProfMucWin*)wins_new_muc(roomjid);
+        mucwin = mucwin_new(roomjid);
     }
     ProfWin *window = (ProfWin*)mucwin;
 
@@ -812,6 +810,7 @@ void
 ui_room_destroy(const char *const roomjid)
 {
     ProfWin *window = (ProfWin*)wins_get_muc(roomjid);
+    GList *privwins = wins_get_private_chats(roomjid);
     if (window == NULL) {
         log_error("Received room destroy result, but no window open for %s.", roomjid);
     } else {
@@ -820,7 +819,6 @@ ui_room_destroy(const char *const roomjid)
         cons_show("Room destroyed: %s", roomjid);
     }
 
-    GList *privwins = wins_get_private_chats(roomjid);
     GList *curr = privwins;
     while (curr) {
         ProfPrivateWin *privwin = curr->data;
@@ -834,12 +832,12 @@ void
 ui_leave_room(const char *const roomjid)
 {
     ProfWin *window = (ProfWin*)wins_get_muc(roomjid);
+    GList *privwins = wins_get_private_chats(roomjid);
     if (window) {
         int num = wins_get_num(window);
         ui_close_win(num);
     }
 
-    GList *privwins = wins_get_private_chats(roomjid);
     GList *curr = privwins;
     while (curr) {
         ProfPrivateWin *privwin = curr->data;
@@ -855,6 +853,7 @@ ui_room_destroyed(const char *const roomjid, const char *const reason, const cha
     const char *const password)
 {
     ProfWin *window = (ProfWin*)wins_get_muc(roomjid);
+    GList *privwins = wins_get_private_chats(roomjid);
     if (window == NULL) {
         log_error("Received room destroy, but no window open for %s.", roomjid);
     } else {
@@ -877,7 +876,6 @@ ui_room_destroyed(const char *const roomjid, const char *const reason, const cha
         }
     }
 
-    GList *privwins = wins_get_private_chats(roomjid);
     GList *curr = privwins;
     while (curr) {
         ProfPrivateWin *privwin = curr->data;
@@ -891,6 +889,7 @@ void
 ui_room_kicked(const char *const roomjid, const char *const actor, const char *const reason)
 {
     ProfWin *window = (ProfWin*)wins_get_muc(roomjid);
+    GList *privwins = wins_get_private_chats(roomjid);
     if (window == NULL) {
         log_error("Received kick, but no window open for %s.", roomjid);
     } else {
@@ -913,7 +912,6 @@ ui_room_kicked(const char *const roomjid, const char *const actor, const char *c
         g_string_free(message, TRUE);
     }
 
-    GList *privwins = wins_get_private_chats(roomjid);
     GList *curr = privwins;
     while (curr) {
         ProfPrivateWin *privwin = curr->data;
@@ -927,6 +925,7 @@ void
 ui_room_banned(const char *const roomjid, const char *const actor, const char *const reason)
 {
     ProfWin *window = (ProfWin*)wins_get_muc(roomjid);
+    GList *privwins = wins_get_private_chats(roomjid);
     if (window == NULL) {
         log_error("Received ban, but no window open for %s.", roomjid);
     } else {
@@ -949,7 +948,6 @@ ui_room_banned(const char *const roomjid, const char *const actor, const char *c
         g_string_free(message, TRUE);
     }
 
-    GList *privwins = wins_get_private_chats(roomjid);
     GList *curr = privwins;
     while (curr) {
         ProfPrivateWin *privwin = curr->data;
