@@ -2,6 +2,7 @@
  * mucwin.c
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2019 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -44,6 +45,27 @@
 #include "ui/window.h"
 #include "ui/win_types.h"
 #include "ui/window_list.h"
+#ifdef HAVE_OMEMO
+#include "omemo/omemo.h"
+#endif
+
+ProfMucWin*
+mucwin_new(const char *const barejid)
+{
+    ProfWin *window = wins_new_muc(barejid);
+    ProfMucWin *mucwin = (ProfMucWin *)window;
+
+    mucwin->last_msg_timestamp = NULL;
+
+#ifdef HAVE_OMEMO
+    if (muc_anonymity_type(mucwin->roomjid) == MUC_ANONYMITY_TYPE_NONANONYMOUS && omemo_automatic_start(barejid)) {
+        omemo_start_muc_sessions(barejid);
+        mucwin->is_omemo = TRUE;
+    }
+#endif
+
+    return mucwin;
+}
 
 void
 mucwin_role_change(ProfMucWin *mucwin, const char *const role, const char *const actor, const char *const reason)
@@ -361,7 +383,7 @@ mucwin_history(ProfMucWin *mucwin, const char *const nick, GDateTime *timestamp,
 }
 
 static void
-_mucwin_print_mention(ProfWin *window, const char *const message, const char *const nick, GSList *mentions)
+_mucwin_print_mention(ProfWin *window, const char *const message, const char *const from, const char *const mynick, GSList *mentions, const char *const ch, int flags)
 {
     int last_pos = 0;
     int pos = 0;
@@ -370,13 +392,20 @@ _mucwin_print_mention(ProfWin *window, const char *const message, const char *co
         pos = GPOINTER_TO_INT(curr->data);
 
         char *before_str = g_strndup(message + last_pos, pos - last_pos);
-        win_append_highlight(window, THEME_ROOMMENTION, "%s", before_str);
+        if (strncmp(before_str, "/me ", 4) == 0) {
+            win_print_them(window, THEME_ROOMMENTION, *ch, flags, "");
+            win_append_highlight(window, THEME_ROOMMENTION, "*%s ", from);
+            win_append_highlight(window, THEME_ROOMMENTION, "%s", before_str + 4);
+        } else {
+            win_print_them(window, THEME_ROOMMENTION, *ch, flags, from);
+            win_append_highlight(window, THEME_ROOMMENTION, "%s", before_str);
+        }
         g_free(before_str);
-        char *nick_str = g_strndup(message + pos, strlen(nick));
-        win_append_highlight(window, THEME_ROOMMENTION_TERM, "%s", nick_str);
-        g_free(nick_str);
+        char *mynick_str = g_strndup(message + pos, strlen(mynick));
+        win_append_highlight(window, THEME_ROOMMENTION_TERM, "%s", mynick_str);
+        g_free(mynick_str);
 
-        last_pos = pos + strlen(nick);
+        last_pos = pos + strlen(mynick);
 
         curr = g_slist_next(curr);
     }
@@ -471,30 +500,65 @@ _mucwin_print_triggers(ProfWin *window, const char *const message, GList *trigge
 }
 
 void
-mucwin_message(ProfMucWin *mucwin, const char *const nick, const char *const message, GSList *mentions, GList *triggers)
+mucwin_outgoing_msg(ProfMucWin *mucwin, const char *const message, const char *const id, prof_enc_t enc_mode)
 {
     assert(mucwin != NULL);
 
+    g_hash_table_insert(mucwin->sent_messages, strdup(id), NULL);
+
     ProfWin *window = (ProfWin*)mucwin;
     char *mynick = muc_nick(mucwin->roomjid);
 
     char ch = '-';
     if (mucwin->message_char) {
         ch = mucwin->message_char[0];
+    } else if (enc_mode == PROF_MSG_ENC_OTR) {
+        ch = prefs_get_otr_char();
+    } else if (enc_mode == PROF_MSG_ENC_PGP) {
+        ch = prefs_get_pgp_char();
+    } else if (enc_mode == PROF_MSG_ENC_OMEMO) {
+        ch = prefs_get_omemo_char();
     }
 
-    if (g_strcmp0(nick, mynick) != 0) {
-        if (g_slist_length(mentions) > 0) {
-            win_print_them(window, THEME_ROOMMENTION, ch, nick);
-            _mucwin_print_mention(window, message, mynick, mentions);
-        } else if (triggers) {
-            win_print_them(window, THEME_ROOMTRIGGER, ch, nick);
-            _mucwin_print_triggers(window, message, triggers);
-        } else {
-            win_println_them_message(window, ch, nick, "%s", message);
-        }
+    win_println_me_message(window, ch, mynick, "%s", message);
+}
+
+void
+mucwin_incoming_msg(ProfMucWin *mucwin, ProfMessage *message, GSList *mentions, GList *triggers)
+{
+    assert(mucwin != NULL);
+    int flags = 0;
+
+    if (message->id && g_hash_table_remove(mucwin->sent_messages, message->id)) {
+        /* Ignore reflection messages */
+        return;
+    }
+
+    if (!message->trusted) {
+        flags |= UNTRUSTED;
+    }
+
+    ProfWin *window = (ProfWin*)mucwin;
+    char *mynick = muc_nick(mucwin->roomjid);
+
+    char ch = '-';
+    if (mucwin->message_char) {
+        ch = mucwin->message_char[0];
+    } else if (message->enc == PROF_MSG_ENC_OTR) {
+        ch = prefs_get_otr_char();
+    } else if (message->enc == PROF_MSG_ENC_PGP) {
+        ch = prefs_get_pgp_char();
+    } else if (message->enc == PROF_MSG_ENC_OMEMO) {
+        ch = prefs_get_omemo_char();
+    }
+
+    if (g_slist_length(mentions) > 0) {
+        _mucwin_print_mention(window, message->plain, message->jid->resourcepart, mynick, mentions, &ch, flags);
+    } else if (triggers) {
+        win_print_them(window, THEME_ROOMTRIGGER, ch, flags, message->jid->resourcepart);
+        _mucwin_print_triggers(window, message->plain, triggers);
     } else {
-        win_println_me_message(window, ch, mynick, "%s", message);
+        win_println_them_message(window, ch, flags, message->jid->resourcepart, "%s", message->plain);
     }
 }
 
