@@ -561,6 +561,11 @@ wins_close_by_num(int i)
                 ProfMucWin *mucwin = (ProfMucWin*)window;
                 autocomplete_remove(wins_ac, mucwin->roomjid);
                 autocomplete_remove(wins_close_ac, mucwin->roomjid);
+                g_hash_table_remove_all(mucwin->sent_messages);
+
+                if (mucwin->last_msg_timestamp) {
+                    g_date_time_unref(mucwin->last_msg_timestamp);
+                }
                 break;
             }
             case WIN_PRIVATE:
@@ -844,6 +849,27 @@ wins_lost_connection(void)
     g_list_free(values);
 }
 
+void
+wins_reestablished_connection(void)
+{
+    GList *values = g_hash_table_get_values(windows);
+    GList *curr = values;
+
+    while (curr) {
+        ProfWin *window = curr->data;
+        if (window->type != WIN_CONSOLE) {
+            win_println(window, THEME_TEXT, '-', "Connection re-established.");
+
+            // if current win, set current_win_dirty
+            if (wins_is_current(window)) {
+                win_update_virtual(window);
+            }
+        }
+        curr = g_list_next(curr);
+    }
+    g_list_free(values);
+}
+
 void
 wins_swap(int source_win, int target_win)
 {
