@@ -57,12 +57,15 @@ typedef struct prof_conn_t {
     xmpp_log_t *xmpp_log;
     xmpp_ctx_t *xmpp_ctx;
     xmpp_conn_t *xmpp_conn;
+    gboolean xmpp_in_event_loop;
     jabber_conn_status_t conn_status;
+    xmpp_conn_event_t conn_last_event;
     char *presence_message;
     int priority;
     char *domain;
     GHashTable *available_resources;
     GHashTable *features_by_jid;
+    GHashTable *requested_features;
 } ProfConnection;
 
 static ProfConnection conn;
@@ -84,17 +87,22 @@ connection_init(void)
     xmpp_initialize();
     conn.xmpp_conn = NULL;
     conn.xmpp_ctx = NULL;
+    conn.xmpp_in_event_loop = FALSE;
     conn.conn_status = JABBER_DISCONNECTED;
+    conn.conn_last_event = XMPP_CONN_DISCONNECT;
     conn.presence_message = NULL;
     conn.domain = NULL;
     conn.features_by_jid = NULL;
     conn.available_resources = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)resource_destroy);
+    conn.requested_features = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
 }
 
 void
 connection_check_events(void)
 {
+    conn.xmpp_in_event_loop = TRUE;
     xmpp_run_once(conn.xmpp_ctx, 10);
+    conn.xmpp_in_event_loop = FALSE;
 }
 
 void
@@ -194,21 +202,30 @@ connection_connect(const char *const jid, const char *const passwd, const char *
 void
 connection_disconnect(void)
 {
-    conn.conn_status = JABBER_DISCONNECTING;
-    xmpp_disconnect(conn.xmpp_conn);
+    // don't disconnect already disconnected connection,
+    // or we get infinite loop otherwise
+    if (conn.conn_last_event == XMPP_CONN_CONNECT) {
+        conn.conn_status = JABBER_DISCONNECTING;
+        xmpp_disconnect(conn.xmpp_conn);
 
-    while (conn.conn_status == JABBER_DISCONNECTING) {
-        session_process_events();
+        while (conn.conn_status == JABBER_DISCONNECTING) {
+            session_process_events();
+        }
+    } else {
+        conn.conn_status = JABBER_DISCONNECTED;
     }
 
-    if (conn.xmpp_conn) {
-        xmpp_conn_release(conn.xmpp_conn);
-        conn.xmpp_conn = NULL;
-    }
+    // can't free libstrophe objects while we're in the event loop
+    if (!conn.xmpp_in_event_loop) {
+        if (conn.xmpp_conn) {
+            xmpp_conn_release(conn.xmpp_conn);
+            conn.xmpp_conn = NULL;
+        }
 
-    if (conn.xmpp_ctx) {
-        xmpp_ctx_free(conn.xmpp_ctx);
-        conn.xmpp_ctx = NULL;
+        if (conn.xmpp_ctx) {
+            xmpp_ctx_free(conn.xmpp_ctx);
+            conn.xmpp_ctx = NULL;
+        }
     }
 }
 
@@ -231,6 +248,10 @@ connection_clear_data(void)
     if (conn.available_resources) {
         g_hash_table_remove_all(conn.available_resources);
     }
+
+    if (conn.requested_features) {
+        g_hash_table_remove_all(conn.requested_features);
+    }
 }
 
 #ifdef HAVE_LIBMESODE
@@ -269,6 +290,7 @@ connection_send_stanza(const char *const stanza)
 gboolean
 connection_supports(const char *const feature)
 {
+    gboolean ret = FALSE;
     GList *jids = g_hash_table_get_keys(conn.features_by_jid);
 
     GList *curr = jids;
@@ -276,7 +298,8 @@ connection_supports(const char *const feature)
         char *jid = curr->data;
         GHashTable *features = g_hash_table_lookup(conn.features_by_jid, jid);
         if (features && g_hash_table_lookup(features, feature)) {
-            return TRUE;
+            ret = TRUE;
+            break;
         }
 
         curr = g_list_next(curr);
@@ -284,7 +307,7 @@ connection_supports(const char *const feature)
 
     g_list_free(jids);
 
-    return FALSE;
+    return ret;
 }
 
 char*
@@ -313,12 +336,21 @@ connection_jid_for_feature(const char *const feature)
     return NULL;
 }
 
+void
+connection_request_features(void)
+{
+    /* We don't record it as a requested feature to avoid triggering th
+     * sv_ev_connection_features_received too soon */
+    iq_disco_info_request_onconnect(conn.domain);
+}
+
 void
 connection_set_disco_items(GSList *items)
 {
     GSList *curr = items;
     while (curr) {
         DiscoItem *item = curr->data;
+        g_hash_table_insert(conn.requested_features, strdup(item->jid), NULL);
         g_hash_table_insert(conn.features_by_jid, strdup(item->jid),
             g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL));
 
@@ -357,6 +389,14 @@ connection_get_fulljid(void)
     }
 }
 
+void
+connection_features_received(const char *const jid)
+{
+    if (g_hash_table_remove(conn.requested_features, jid) && g_hash_table_size(conn.requested_features) == 0) {
+        sv_ev_connection_features_received();
+    }
+}
+
 GHashTable*
 connection_get_features(const char *const jid)
 {
@@ -447,6 +487,8 @@ static void
 _connection_handler(xmpp_conn_t *const xmpp_conn, const xmpp_conn_event_t status, const int error,
     xmpp_stream_error_t *const stream_error, void *const userdata)
 {
+    conn.conn_last_event = status;
+
     switch (status) {
 
     // login success
