@@ -72,16 +72,20 @@
 #include "xmpp/roster.h"
 #include "xmpp/muc.h"
 
+#ifdef HAVE_OMEMO
+#include "omemo/omemo.h"
+#endif
+
 typedef struct p_room_info_data_t {
     char *room;
     gboolean display;
 } ProfRoomInfoData;
 
-typedef struct p_id_handle_t {
-    ProfIdCallback func;
-    ProfIdFreeCallback free_func;
+typedef struct p_iq_handle_t {
+    ProfIqCallback func;
+    ProfIqFreeCallback free_func;
     void *userdata;
-} ProfIdHandler;
+} ProfIqHandler;
 
 typedef struct privilege_set_t {
     char *item;
@@ -130,6 +134,7 @@ static int _command_exec_response_handler(xmpp_stanza_t *const stanza, void *con
 
 static void _iq_free_room_data(ProfRoomInfoData *roominfo);
 static void _iq_free_affiliation_set(ProfPrivilegeSet *affiliation_set);
+static void _iq_id_handler_free(ProfIqHandler *handler);
 
 // scheduled
 static int _autoping_timed_send(xmpp_conn_t *const conn, void *const userdata);
@@ -205,11 +210,10 @@ _iq_handler(xmpp_conn_t *const conn, xmpp_stanza_t *const stanza, void *const us
 
     const char *id = xmpp_stanza_get_id(stanza);
     if (id) {
-        ProfIdHandler *handler = g_hash_table_lookup(id_handlers, id);
+        ProfIqHandler *handler = g_hash_table_lookup(id_handlers, id);
         if (handler) {
             int keep = handler->func(stanza, handler->userdata);
             if (!keep) {
-                free(handler);
                 g_hash_table_remove(id_handlers, id);
             }
         }
@@ -230,27 +234,38 @@ iq_handlers_init(void)
         xmpp_timed_handler_add(conn, _autoping_timed_send, millis, ctx);
     }
 
-    if (id_handlers) {
-        GList *keys = g_hash_table_get_keys(id_handlers);
-        GList *curr = keys;
-        while (curr) {
-            ProfIdHandler *handler = g_hash_table_lookup(id_handlers, curr->data);
-            if (handler->free_func && handler->userdata) {
-                handler->free_func(handler->userdata);
-            }
-            curr = g_list_next(curr);
-        }
-        g_list_free(keys);
-        g_hash_table_destroy(id_handlers);
-    }
-    id_handlers = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
+    iq_handlers_clear();
+
+    id_handlers = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)_iq_id_handler_free);
     rooms_cache = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)xmpp_stanza_release);
 }
 
 void
-iq_id_handler_add(const char *const id, ProfIdCallback func, ProfIdFreeCallback free_func, void *userdata)
+iq_handlers_clear()
 {
-    ProfIdHandler *handler = malloc(sizeof(ProfIdHandler));
+    if (id_handlers) {
+        g_hash_table_remove_all(id_handlers);
+        id_handlers = NULL;
+    }
+}
+
+static void
+_iq_id_handler_free(ProfIqHandler *handler)
+{
+    if (handler == NULL) {
+        return;
+    }
+    if (handler->free_func && handler->userdata) {
+        handler->free_func(handler->userdata);
+    }
+    free(handler);
+    handler = NULL;
+}
+
+void
+iq_id_handler_add(const char *const id, ProfIqCallback func, ProfIqFreeCallback free_func, void *userdata)
+{
+    ProfIqHandler *handler = malloc(sizeof(ProfIqHandler));
     handler->func = func;
     handler->free_func = free_func;
     handler->userdata = userdata;
@@ -258,6 +273,16 @@ iq_id_handler_add(const char *const id, ProfIdCallback func, ProfIdFreeCallback
     g_hash_table_insert(id_handlers, strdup(id), handler);
 }
 
+void
+iq_autoping_timer_cancel(void)
+{
+    autoping_wait = FALSE;
+    if (autoping_time) {
+        g_timer_destroy(autoping_time);
+        autoping_time = NULL;
+    }
+}
+
 void
 iq_autoping_check(void)
 {
@@ -279,10 +304,8 @@ iq_autoping_check(void)
     if (timeout > 0 && seconds_elapsed >= timeout) {
         cons_show("Autoping response timed out after %u seconds.", timeout);
         log_debug("Autoping check: timed out after %u seconds, disconnecting", timeout);
+        iq_autoping_timer_cancel();
         session_autoping_fail();
-        autoping_wait = FALSE;
-        g_timer_destroy(autoping_time);
-        autoping_time = NULL;
     }
 }
 
@@ -310,6 +333,7 @@ iq_rooms_cache_clear(void)
 {
     if (rooms_cache) {
         g_hash_table_remove_all(rooms_cache);
+        rooms_cache = NULL;
     }
 }
 
@@ -438,7 +462,7 @@ iq_room_info_request(const char *const room, gboolean display_result)
     cb_data->room = strdup(room);
     cb_data->display = display_result;
 
-    iq_id_handler_add(id, _room_info_response_id_handler, (ProfIdFreeCallback)_iq_free_room_data, cb_data);
+    iq_id_handler_add(id, _room_info_response_id_handler, (ProfIqFreeCallback)_iq_free_room_data, cb_data);
 
     free(id);
 
@@ -651,7 +675,7 @@ iq_room_affiliation_set(const char *const room, const char *const jid, char *aff
     affiliation_set->item = strdup(jid);
     affiliation_set->privilege = strdup(affiliation);
 
-    iq_id_handler_add(id, _room_affiliation_set_result_id_handler, (ProfIdFreeCallback)_iq_free_affiliation_set, affiliation_set);
+    iq_id_handler_add(id, _room_affiliation_set_result_id_handler, (ProfIqFreeCallback)_iq_free_affiliation_set, affiliation_set);
 
     iq_send_stanza(iq);
     xmpp_stanza_release(iq);
@@ -670,7 +694,7 @@ iq_room_role_set(const char *const room, const char *const nick, char *role,
     role_set->item = strdup(nick);
     role_set->privilege = strdup(role);
 
-    iq_id_handler_add(id, _room_role_set_result_id_handler, (ProfIdFreeCallback)_iq_free_affiliation_set, role_set);
+    iq_id_handler_add(id, _room_role_set_result_id_handler, (ProfIqFreeCallback)_iq_free_affiliation_set, role_set);
 
     iq_send_stanza(iq);
     xmpp_stanza_release(iq);
@@ -697,7 +721,7 @@ iq_send_ping(const char *const target)
     const char *id = xmpp_stanza_get_id(iq);
 
     GDateTime *now = g_date_time_new_now_local();
-    iq_id_handler_add(id, _manual_pong_id_handler, (ProfIdFreeCallback)g_date_time_unref, now);
+    iq_id_handler_add(id, _manual_pong_id_handler, (ProfIqFreeCallback)g_date_time_unref, now);
 
     iq_send_stanza(iq);
     xmpp_stanza_release(iq);
@@ -859,7 +883,6 @@ _caps_response_for_jid_id_handler(xmpp_stanza_t *const stanza, void *const userd
     const char *type = xmpp_stanza_get_type(stanza);
     // ignore non result
     if ((g_strcmp0(type, "get") == 0) || (g_strcmp0(type, "set") == 0)) {
-        free(jid);
         return 1;
     }
 
@@ -872,7 +895,6 @@ _caps_response_for_jid_id_handler(xmpp_stanza_t *const stanza, void *const userd
     const char *from = xmpp_stanza_get_from(stanza);
     if (!from) {
         log_info("No from attribute");
-        free(jid);
         return 0;
     }
 
@@ -881,20 +903,17 @@ _caps_response_for_jid_id_handler(xmpp_stanza_t *const stanza, void *const userd
         char *error_message = stanza_get_error_message(stanza);
         log_warning("Error received for capabilities response from %s: ", from, error_message);
         free(error_message);
-        free(jid);
         return 0;
     }
 
     if (query == NULL) {
         log_info("No query element found.");
-        free(jid);
         return 0;
     }
 
     const char *node = xmpp_stanza_get_attribute(query, STANZA_ATTR_NODE);
     if (node == NULL) {
         log_info("No node attribute found");
-        free(jid);
         return 0;
     }
 
@@ -902,8 +921,6 @@ _caps_response_for_jid_id_handler(xmpp_stanza_t *const stanza, void *const userd
     EntityCapabilities *capabilities = stanza_create_caps_from_query_element(query);
     caps_add_by_jid(jid, capabilities);
 
-    free(jid);
-
     return 0;
 }
 
@@ -917,7 +934,6 @@ _caps_response_legacy_id_handler(xmpp_stanza_t *const stanza, void *const userda
     const char *type = xmpp_stanza_get_type(stanza);
     // ignore non result
     if ((g_strcmp0(type, "get") == 0) || (g_strcmp0(type, "set") == 0)) {
-        free(expected_node);
         return 1;
     }
 
@@ -930,7 +946,6 @@ _caps_response_legacy_id_handler(xmpp_stanza_t *const stanza, void *const userda
     const char *from = xmpp_stanza_get_from(stanza);
     if (!from) {
         log_info("No from attribute");
-        free(expected_node);
         return 0;
     }
 
@@ -939,20 +954,17 @@ _caps_response_legacy_id_handler(xmpp_stanza_t *const stanza, void *const userda
         char *error_message = stanza_get_error_message(stanza);
         log_warning("Error received for capabilities response from %s: ", from, error_message);
         free(error_message);
-        free(expected_node);
         return 0;
     }
 
     if (query == NULL) {
         log_info("No query element found.");
-        free(expected_node);
         return 0;
     }
 
     const char *node = xmpp_stanza_get_attribute(query, STANZA_ATTR_NODE);
     if (node == NULL) {
         log_info("No node attribute found");
-        free(expected_node);
         return 0;
     }
 
@@ -975,7 +987,6 @@ _caps_response_legacy_id_handler(xmpp_stanza_t *const stanza, void *const userda
         log_info("Legacy Capabilities nodes do not match, expeceted %s, given %s.", expected_node, node);
     }
 
-    free(expected_node);
     return 0;
 }
 
@@ -994,7 +1005,6 @@ _room_list_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
 
     xmpp_stanza_t *query = xmpp_stanza_get_child_by_name(stanza, STANZA_NAME_QUERY);
     if (query == NULL) {
-        g_free(filter);
         return 0;
     }
 
@@ -1007,7 +1017,6 @@ _room_list_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
     xmpp_stanza_t *child = xmpp_stanza_get_children(query);
     if (child == NULL) {
         cons_show("  No rooms found.");
-        g_free(filter);
         return 0;
     }
 
@@ -1016,7 +1025,6 @@ _room_list_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
         gchar *filter_lower = g_utf8_strdown(filter, -1);
         GString *glob_str = g_string_new("*");
         g_string_append(glob_str, filter_lower);
-        g_free(filter_lower);
         g_string_append(glob_str, "*");
         glob = g_pattern_spec_new(glob_str->str);
         g_string_free(glob_str, TRUE);
@@ -1069,7 +1077,6 @@ _room_list_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
     if (glob) {
         g_pattern_spec_free(glob);
     }
-    g_free(filter);
 
     return 0;
 }
@@ -1235,7 +1242,7 @@ _command_exec_response_handler(xmpp_stanza_t *const stanza, void *const userdata
         } else {
             data->sessionid = strdup(sessionid);
         }
-        data->command = command;
+        data->command = strdup(command);
         ProfConfWin *confwin = (ProfConfWin*)wins_new_config(from, form, iq_submit_command_config, iq_cancel_command_config, data);
         confwin_handle_configuration(confwin, form);
     } else if (g_strcmp0(status, "canceled") == 0) {
@@ -1303,7 +1310,6 @@ _manual_pong_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
         }
 
         free(error_message);
-        g_date_time_unref(sent);
         return 0;
     }
 
@@ -1312,7 +1318,6 @@ _manual_pong_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
     GTimeSpan elapsed = g_date_time_difference(now, sent);
     int elapsed_millis = elapsed / 1000;
 
-    g_date_time_unref(sent);
     g_date_time_unref(now);
 
     if (from == NULL) {
@@ -1367,11 +1372,7 @@ _autoping_timed_send(xmpp_conn_t *const conn, void *const userdata)
 static int
 _auto_pong_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
 {
-    autoping_wait = FALSE;
-    if (autoping_time) {
-        g_timer_destroy(autoping_time);
-        autoping_time = NULL;
-    }
+    iq_autoping_timer_cancel();
 
     const char *id = xmpp_stanza_get_id(stanza);
     if (id == NULL) {
@@ -1504,7 +1505,6 @@ _version_result_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
     ui_show_software_version(jidp->fulljid, presence, name_str, version_str, os_str);
 
     jid_destroy(jidp);
-    free(userdata);
 
     if (name_str) xmpp_free(ctx, name_str);
     if (version_str) xmpp_free(ctx, version_str);
@@ -1814,10 +1814,6 @@ _room_affiliation_set_result_id_handler(xmpp_stanza_t *const stanza, void *const
         free(error_message);
     }
 
-    free(affiliation_set->item);
-    free(affiliation_set->privilege);
-    free(affiliation_set);
-
     return 0;
 }
 
@@ -1846,10 +1842,6 @@ _room_role_set_result_id_handler(xmpp_stanza_t *const stanza, void *const userda
         free(error_message);
     }
 
-    free(role_set->item);
-    free(role_set->privilege);
-    free(role_set);
-
     return 0;
 }
 
@@ -1876,7 +1868,6 @@ _room_affiliation_list_result_id_handler(xmpp_stanza_t *const stanza, void *cons
             mucwin_affiliation_list_error(mucwin, affiliation, error_message);
         }
         free(error_message);
-        free(affiliation);
         return 0;
     }
     GSList *jids = NULL;
@@ -1901,7 +1892,6 @@ _room_affiliation_list_result_id_handler(xmpp_stanza_t *const stanza, void *cons
     if (mucwin) {
         mucwin_handle_affiliation_list(mucwin, affiliation, jids);
     }
-    free(affiliation);
     g_slist_free(jids);
 
     return 0;
@@ -1930,7 +1920,6 @@ _room_role_list_result_id_handler(xmpp_stanza_t *const stanza, void *const userd
             mucwin_role_list_error(mucwin, role, error_message);
         }
         free(error_message);
-        free(role);
         return 0;
     }
     GSList *nicks = NULL;
@@ -1954,7 +1943,6 @@ _room_role_list_result_id_handler(xmpp_stanza_t *const stanza, void *const userd
     if (mucwin) {
         mucwin_handle_role_list(mucwin, role, nicks);
     }
-    free(role);
     g_slist_free(nicks);
 
     return 0;
@@ -2008,8 +1996,6 @@ _room_kick_result_id_handler(xmpp_stanza_t *const stanza, void *const userdata)
         free(error_message);
     }
 
-    free(nick);
-
     return 0;
 }
 
@@ -2049,8 +2035,6 @@ _room_info_response_id_handler(xmpp_stanza_t *const stanza, void *const userdata
             mucwin_room_info_error(mucwin, error_message);
             free(error_message);
         }
-        free(cb_data->room);
-        free(cb_data);
         return 0;
     }
 
@@ -2100,17 +2084,22 @@ _room_info_response_id_handler(xmpp_stanza_t *const stanza, void *const userdata
 
         muc_set_features(cb_data->room, features);
         ProfMucWin *mucwin = wins_get_muc(cb_data->room);
-        if (mucwin && cb_data->display) {
-            mucwin_room_disco_info(mucwin, identities, features);
+        if (mucwin) {
+#ifdef HAVE_OMEMO
+            if (muc_anonymity_type(mucwin->roomjid) == MUC_ANONYMITY_TYPE_NONANONYMOUS && omemo_automatic_start(cb_data->room)) {
+                omemo_start_muc_sessions(cb_data->room);
+                mucwin->is_omemo = TRUE;
+            }
+#endif
+            if (cb_data->display) {
+                mucwin_room_disco_info(mucwin, identities, features);
+            }
         }
 
         g_slist_free_full(features, free);
         g_slist_free_full(identities, (GDestroyNotify)_identity_destroy);
     }
 
-    free(cb_data->room);
-    free(cb_data);
-
     return 0;
 }
 
@@ -2291,6 +2280,8 @@ _disco_info_response_id_handler_onconnect(xmpp_stanza_t *const stanza, void *con
         }
     }
 
+    connection_features_received(from);
+
     return 0;
 }
 
@@ -2418,8 +2409,8 @@ iq_send_stanza(xmpp_stanza_t *const stanza)
         xmpp_send_raw_string(conn, "%s", text);
     }
     xmpp_free(connection_get_ctx(), text);
-
 }
+
 static void
 _iq_free_room_data(ProfRoomInfoData *roominfo)
 {
