@@ -1,7 +1,7 @@
 /*
  * presence.c
  *
- * Copyright (C) 2012 - 2017 James Booth <boothj5@gmail.com>
+ * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  *
  * This file is part of Profanity.
  *
@@ -128,7 +128,7 @@ presence_subscription(const char *const jid, const jabber_subscr_t action)
     xmpp_ctx_t * const ctx = connection_get_ctx();
     xmpp_stanza_t *presence = xmpp_presence_new(ctx);
 
-    char *id = create_unique_id("sub");
+    char *id = connection_create_stanza_id("sub");
     xmpp_stanza_set_id(presence, id);
     free(id);
 
@@ -141,7 +141,7 @@ presence_subscription(const char *const jid, const jabber_subscr_t action)
     xmpp_stanza_release(presence);
 }
 
-GSList*
+GList*
 presence_get_subscription_requests(void)
 {
     return autocomplete_create_list(sub_requests_ac);
@@ -160,9 +160,9 @@ presence_clear_sub_requests(void)
 }
 
 char*
-presence_sub_request_find(const char *const search_str)
+presence_sub_request_find(const char *const search_str, gboolean previous)
 {
-    return autocomplete_complete(sub_requests_ac, search_str, TRUE);
+    return autocomplete_complete(sub_requests_ac, search_str, TRUE, previous);
 }
 
 gboolean
@@ -170,16 +170,16 @@ presence_sub_request_exists(const char *const bare_jid)
 {
     gboolean result = FALSE;
 
-    GSList *requests = autocomplete_create_list(sub_requests_ac);
-    GSList *curr = requests;
+    GList *requests = autocomplete_create_list(sub_requests_ac);
+    GList *curr = requests;
     while (curr) {
         if (strcmp(curr->data, bare_jid) == 0) {
             result = TRUE;
             break;
         }
-        curr = g_slist_next(curr);
+        curr = g_list_next(curr);
     }
-    g_slist_free_full(requests, free);
+    g_list_free_full(requests, free);
 
     return result;
 }
@@ -211,7 +211,7 @@ presence_send(const resource_presence_t presence_type, const int idle, char *sig
     xmpp_ctx_t * const ctx = connection_get_ctx();
     xmpp_stanza_t *presence = xmpp_presence_new(ctx);
 
-    char *id = create_unique_id("presence");
+    char *id = connection_create_stanza_id("presence");
     xmpp_stanza_set_id(presence, id);
     free(id);
 
@@ -579,7 +579,7 @@ _handle_caps(const char *const jid, XMPPCaps *caps)
                 caps_map_jid_to_ver(jid, caps->ver);
             } else {
                 log_info("Capabilities cache miss: %s, for %s, sending service discovery request", caps->ver, jid);
-                char *id = create_unique_id("caps");
+                char *id = connection_create_stanza_id("caps");
                 iq_send_caps_request(jid, id, caps->node, caps->ver);
                 free(id);
             }
@@ -588,14 +588,14 @@ _handle_caps(const char *const jid, XMPPCaps *caps)
     // unsupported hash, xep-0115, associate with JID, no cache
     } else if (caps->hash) {
         log_info("Hash %s not supported: %s, sending service discovery request", caps->hash, jid);
-        char *id = create_unique_id("caps");
+        char *id = connection_create_stanza_id("caps");
         iq_send_caps_request_for_jid(jid, id, caps->node, caps->ver);
         free(id);
 
    // no hash, legacy caps, cache against node#ver
    } else if (caps->node && caps->ver) {
         log_info("No hash specified: %s, legacy request made for %s#%s", jid, caps->node, caps->ver);
-        char *id = create_unique_id("caps");
+        char *id = connection_create_stanza_id("caps");
         iq_send_caps_request_legacy(jid, id, caps->node, caps->ver);
         free(id);
     } else {
