@@ -67,7 +67,15 @@ typedef struct prof_roster_t {
     GHashTable *group_count;
 } ProfRoster;
 
+typedef struct pending_presence {
+    char *barejid;
+    Resource *resource;
+    GDateTime *last_activity;
+} ProfPendingPresence;
+
 static ProfRoster *roster = NULL;
+static gboolean roster_received = FALSE;
+static GSList *roster_pending_presence = NULL;
 
 static gboolean _key_equals(void *key1, void *key2);
 static gboolean _datetimes_equal(GDateTime *dt1, GDateTime *dt2);
@@ -87,6 +95,9 @@ roster_create(void)
     roster->name_to_barejid = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
     roster->groups_ac = autocomplete_new();
     roster->group_count = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+
+    roster_received = FALSE;
+    roster_pending_presence = NULL;
 }
 
 void
@@ -114,6 +125,18 @@ roster_update_presence(const char *const barejid, Resource *resource, GDateTime
     assert(barejid != NULL);
     assert(resource != NULL);
 
+    if (!roster_received) {
+        ProfPendingPresence *presence = malloc(sizeof(ProfPendingPresence));
+        presence->barejid = strdup(barejid);
+        presence->resource = resource;
+        presence->last_activity = last_activity;
+        if (last_activity) {
+            g_date_time_ref(last_activity);
+        }
+        roster_pending_presence = g_slist_append(roster_pending_presence, presence);
+        return FALSE;
+    }
+
     PContact contact = roster_get_contact(barejid);
     if (contact == NULL) {
         return FALSE;
@@ -209,10 +232,9 @@ void
 roster_change_name(PContact contact, const char *const new_name)
 {
     assert(roster != NULL);
-
     assert(contact != NULL);
 
-    const char *current_name = NULL;
+    char *current_name = NULL;
     const char *barejid = p_contact_barejid(contact);
 
     if (p_contact_name(contact)) {
@@ -221,6 +243,7 @@ roster_change_name(PContact contact, const char *const new_name)
 
     p_contact_set_name(contact, new_name);
     _replace_name(current_name, new_name, barejid);
+    free(current_name);
 }
 
 void
@@ -237,7 +260,7 @@ roster_remove(const char *const name, const char *const barejid)
     if (contact) {
         GList *resources = p_contact_get_available_resources(contact);
         while (resources) {
-            GString *fulljid = g_string_new(strdup(barejid));
+            GString *fulljid = g_string_new(barejid);
             g_string_append(fulljid, "/");
             g_string_append(fulljid, resources->data);
             autocomplete_remove(roster->fulljid_ac, fulljid->str);
@@ -281,13 +304,14 @@ roster_update(const char *const barejid, const char *const name, GSList *groups,
     p_contact_set_pending_out(contact, pending_out);
 
     const char * const new_name = name;
-    const char * current_name = NULL;
+    char * current_name = NULL;
     if (p_contact_name(contact)) {
         current_name = strdup(p_contact_name(contact));
     }
 
     p_contact_set_name(contact, new_name);
     _replace_name(current_name, new_name, barejid);
+    free(current_name);
 
     GSList *curr_new_group = groups;
     while (curr_new_group) {
@@ -664,3 +688,39 @@ roster_compare_presence(PContact a, PContact b)
         return roster_compare_name(a, b);
     }
 }
+
+static void
+_pendingPresence_free(ProfPendingPresence *presence)
+{
+    if (!presence)
+        return;
+    free(presence->barejid);
+    free(presence);
+}
+
+void
+roster_process_pending_presence(void)
+{
+    roster_received = TRUE;
+
+    GSList *iter;
+    for (iter = roster_pending_presence; iter != NULL; iter = iter->next) {
+        ProfPendingPresence *presence = iter->data;
+        roster_update_presence(presence->barejid, presence->resource, presence->last_activity);
+        /* seems like resource isn't free on the calling side */
+        if (presence->last_activity) {
+            g_date_time_unref(presence->last_activity);
+        }
+    }
+
+    g_slist_free_full(roster_pending_presence, (GDestroyNotify)_pendingPresence_free);
+    roster_pending_presence = NULL;
+}
+
+gboolean
+roster_exists(void) {
+    if (roster != NULL) {
+        return TRUE;
+    }
+    return FALSE;
+}
