@@ -152,17 +152,17 @@ void ui_contact_online(char *barejid, Resource *resource, GDateTime *last_activi
 }
 
 void ui_contact_typing(const char * const barejid, const char * const resource) {}
-void chatwin_incoming_msg(ProfChatWin *chatwin, const char * const resource, const char * const message, GDateTime *timestamp, gboolean win_created, prof_enc_t enc_mode) {}
+void chatwin_incoming_msg(ProfChatWin *chatwin, ProfMessage *message, gboolean win_created) {}
 void chatwin_receipt_received(ProfChatWin *chatwin, const char * const id) {}
 
-void privwin_incoming_msg(ProfPrivateWin *privatewin, const char * const message, GDateTime *timestamp) {}
+void privwin_incoming_msg(ProfPrivateWin *privatewin, ProfMessage *message) {}
 
 void ui_disconnected(void) {}
 void chatwin_recipient_gone(ProfChatWin *chatwin) {}
 
 void chatwin_outgoing_msg(ProfChatWin *chatwin, const char * const message, char *id, prof_enc_t enc_mode,
     gboolean request_receipt) {}
-void chatwin_outgoing_carbon(ProfChatWin *chatwin, const char * const message, prof_enc_t enc_mode) {}
+void chatwin_outgoing_carbon(ProfChatWin *chatwin, ProfMessage *message) {}
 void privwin_outgoing_msg(ProfPrivateWin *privwin, const char * const message) {}
 
 void privwin_occupant_offline(ProfPrivateWin *privwin) {}
@@ -190,7 +190,8 @@ void mucwin_occupant_role_and_affiliation_change(ProfMucWin *mucwin, const char
     const char * const affiliation, const char * const actor, const char * const reason) {}
 void mucwin_roster(ProfMucWin *mucwin, GList *occupants, const char * const presence) {}
 void mucwin_history(ProfMucWin *mucwin, const char * const nick, GDateTime *timestamp, const char * const message) {}
-void mucwin_message(ProfMucWin *mucwin, const char *const nick, const char *const message, GSList *mentions, GList *triggers) {}
+void mucwin_incoming_msg(ProfMucWin *mucwin, ProfMessage *message, GSList *mentions, GList *triggers) {}
+void mucwin_outgoing_msg(ProfMucWin *mucwin, const char *const message, const char *const id, prof_enc_t enc_mode) {}
 void mucwin_subject(ProfMucWin *mucwin, const char * const nick, const char * const subject) {}
 void mucwin_requires_config(ProfMucWin *mucwin) {}
 void ui_room_destroy(const char * const roomjid) {}
@@ -337,6 +338,7 @@ void cons_show_presence_prefs(void) {}
 void cons_show_connection_prefs(void) {}
 void cons_show_otr_prefs(void) {}
 void cons_show_pgp_prefs(void) {}
+void cons_show_omemo_prefs(void) {}
 
 void cons_show_account(ProfAccount *account)
 {
@@ -427,7 +429,6 @@ void cons_wrap_setting(void) {}
 void cons_winstidy_setting(void) {}
 void cons_encwarn_setting(void) {}
 void cons_time_setting(void) {}
-void cons_mouse_setting(void) {}
 void cons_statuses_setting(void) {}
 void cons_wintitle_setting(void) {}
 void cons_notify_setting(void) {}
@@ -476,11 +477,12 @@ void rosterwin_roster(void) {}
 
 // occupants window
 void occupantswin_occupants(const char * const room) {}
+void occupantswin_occupants_all(void) {}
 
 // window interface
 ProfWin* win_create_console(void)
 {
-    return (ProfWin*)mock();
+    return mock_ptr_type(ProfWin*);
 }
 ProfWin* win_create_xmlconsole(void)
 {
@@ -488,7 +490,7 @@ ProfWin* win_create_xmlconsole(void)
 }
 ProfWin* win_create_chat(const char * const barejid)
 {
-    return (ProfWin*)mock();
+    return mock_ptr_type(ProfWin*);
 }
 ProfWin* win_create_muc(const char * const roomjid)
 {
